

/**************************************************************************
 *
 *   DgCollectCallBlock application for Digivoice Cards.
 * 
 *   Based on app_flash.c from Digium(c)
 *
 *   This program is free software, distributed under the terms of
 *   the GNU General Public License Version 2. See the LICENSE file
 *   at the top of the source tree.
 *
 *-------------------------------------------------------------------------*/
 
 
#include "asterisk.h"
#include "config.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 1.1 $")

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <voicerlib/voicerlib.h>

#include "asterisk/lock.h"
#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/translate.h"
#include "asterisk/image.h"
#include "asterisk/options.h"
#include "asterisk/config.h"

#include "dgvchannel.h"

#define AST_MODULE "ael"

#ifndef CC_AST_HAS_VERSION_1_4
static char *tdesc = "Collect Call Block for dgv trunk application";
#endif

static char *app = "DgCollectCallBlock";

static char *synopsis = "Block Collect call on a Dgv Trunk";

static char *descrip = 
"  DgCollectCallBlock(): Block collect call on a Dgv Trunk \n"
".\n";

#ifndef CC_AST_HAS_VERSION_1_4
STANDARD_LOCAL_USER;

LOCAL_USER_DECL;
#endif

static int ccb_exec(struct ast_channel *chan, void *data)
{

    struct dgv_pvt *p = (struct dgv_pvt *)chan->tech_pvt;

	int res = -1;
    //char *parse;

    short pause_before;
    short flash_pause;
    short pause_after;
    short flash_count=1;

#ifdef CC_AST_HAS_VERSION_1_4
    struct ast_module_user *u;
    
    u = ast_module_user_add(chan);
#else
	struct localuser *u;
	
    LOCAL_USER_ADD(u);
#endif

#ifdef CC_AST_HAS_VERSION_1_4
	if (!strcasecmp(chan->tech->type, "Dgv")) {
#else
	if (!strcasecmp(chan->type, "Dgv")) {
#endif
        flash_count = 1;
        pause_before = 1000;
        flash_pause = 2000;
        pause_after = 500;

        p->flashing=1;

        //wait to start flash
        usleep(pause_before*1000);

        //check if channel is still up
        if (!p->ast_ch)
            return -1;

        dg_HangUp(p->port);

        usleep(flash_pause*1000);

        //check if channel is still up
        if (!p->ast_ch)
            return -1;

        dg_PickUp(p->port,100);

        p->flashing=0;

        usleep(1000*pause_after+50);

        //check if channel is still up
        if (!p->ast_ch)
            return -1;

        if (p->ast_ch)
        {
            res = 0;
        }
        else
        {
            res = -1;
        }
                
	} else
		ast_log(LOG_WARNING, "%s is not a Dgv channel\n", chan->name);

#ifdef CC_AST_HAS_VERSION_1_4
    ast_module_user_remove(u);
#else
    LOCAL_USER_REMOVE(u);
#endif
    
	return res;
}

#ifdef CC_AST_HAS_VERSION_1_4	
static int unload_module(void)
#else
int unload_module(void)
#endif
{
	int res;

	res = ast_unregister_application(app);
	
#ifdef CC_AST_HAS_VERSION_1_4	
    ast_module_user_hangup_all();
#else
	STANDARD_HANGUP_LOCALUSERS;
#endif	

	return res;
}

#ifdef CC_AST_HAS_VERSION_1_4	
static int load_module(void)
#else
int load_module(void)
#endif
{
	return ast_register_application(app, ccb_exec, synopsis, descrip);
}

#ifdef CC_AST_HAS_VERSION_1_4
AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Collect Call Block for dgv trunk application");
#else
char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}

#endif
