
/***************************************************************************
 *                                                                         *
 *   app_dgflash.c - DgFlash application for Digivoice Cards.              *
 *                                                                         *
 *   Copyright (c) 2007, Digivoice Tecnologia em Eletronica Ltda           *
 *                                                                         *
 *   Author: Luciano Alves Barroso                                         *
 *   luciano@digivoice.com.br                                              *
 *                                                                         *
 *   This software is licensed under GPL                                   *
 *   Check LICENSE file for details                                        *
 *                                                                         *
 *-------------------------------------------------------------------------*/
 
#include "asterisk.h"
#include "config.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 1.1 $")

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <voicerlib/voicerlib.h>

#include "asterisk/lock.h"
#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/translate.h"
#include "asterisk/image.h"
#include "asterisk/options.h"
#include "asterisk/config.h"

#include "dgvchannel.h"

#define AST_MODULE "ael"

#define AST_DECLARE_APP_ARGS(name, arglist) \
	struct { \
		unsigned int argc; \
		char *argv[0]; \
		arglist \
	} name

#define AST_APP_ARG(name) char *name

#define AST_STANDARD_APP_ARGS(args, parse) \
	args.argc = ast_app_separate_args(parse, '|', args.argv, (sizeof(args) - sizeof(args.argc)) / sizeof(args.argv[0]))

#ifndef CC_AST_HAS_VERSION_1_4
static char *tdesc = "Flash for dgv trunk application";
#endif

static char *app = "DgFlash";

static char *synopsis = "Flash on a Dgv Trunk";

static char *descrip = 
"  DgFlash(flash_pause, pause_after): Flash on a Dgv Trunk \n"
".\n";

#ifndef CC_AST_HAS_VERSION_1_4
STANDARD_LOCAL_USER;

LOCAL_USER_DECL;
#endif

static int flash_exec(struct ast_channel *chan, void *data)
{

    struct dgv_pvt *p = (struct dgv_pvt *)chan->tech_pvt;

	int res = -1;
	char *parse;

    short flash_pause;
    short pause_after;

#ifdef CC_AST_HAS_VERSION_1_4
    struct ast_module_user *u;
    
    u = ast_module_user_add(chan);
#else
	struct localuser *u;
	
    LOCAL_USER_ADD(u);
#endif

    AST_DECLARE_APP_ARGS(args,
                         AST_APP_ARG(flash_pause);
                         AST_APP_ARG(pause_after);
    );


    if (ast_strlen_zero(data)) {
        ast_log(LOG_WARNING, "Flash requires 2 arguments \n");
        return -1;
    }

    if (!(parse = ast_strdupa(data))) {
        ast_log(LOG_WARNING, "Memory allocation failure\n");        
#ifdef CC_AST_HAS_VERSION_1_4
        ast_module_user_remove(u);
#else
        LOCAL_USER_REMOVE(u);
#endif     
        return -1;
    }

    AST_STANDARD_APP_ARGS(args, parse);

    if (args.argc!=2) {
        ast_log(LOG_ERROR, "Wrong arguments\n");
#ifdef CC_AST_HAS_VERSION_1_4
        ast_module_user_remove(u);
#else
        LOCAL_USER_REMOVE(u);
#endif     
        return -1;
    }

    //receive times
    flash_pause = atoi(args.argv[0]);
    pause_after = atoi(args.argv[1]);




#ifdef CC_AST_HAS_VERSION_1_4
	if (!strcasecmp(chan->tech->type, "Dgv")) {
#else
	if (!strcasecmp(chan->type, "Dgv")) {
#endif
        //flash_pause = 250;
        //pause_after = 500;

        p->flashing=1;

        //check if channel is still up
        if (!p->ast_ch)
            return -1;

        dg_HangUp(p->port);

        usleep(flash_pause*1000);

        //check if channel is still up
        if (!p->ast_ch)
            return -1;

        dg_PickUp(p->port,100);

        p->flashing=0;

        usleep(pause_after*1000);

        //check if channel is still up
        if (!p->ast_ch)
            return -1;

        if (p->ast_ch)
        {
            res = 0;
        }
        else
        {
            res = -1;
        }
                
	} else
		ast_log(LOG_WARNING, "%s is not a Dgv channel\n", chan->name);

#ifdef CC_AST_HAS_VERSION_1_4
    ast_module_user_remove(u);
#else
    LOCAL_USER_REMOVE(u);
#endif
    
	return res;
}

#ifdef CC_AST_HAS_VERSION_1_4	
static int unload_module(void)
#else
int unload_module(void)
#endif
{
	int res;

	res = ast_unregister_application(app);
	
#ifdef CC_AST_HAS_VERSION_1_4	
    ast_module_user_hangup_all();
#else
	STANDARD_HANGUP_LOCALUSERS;
#endif	

	return res;
}

#ifdef CC_AST_HAS_VERSION_1_4	
static int load_module(void)
#else
int load_module(void)
#endif
{
	return ast_register_application(app, flash_exec, synopsis, descrip);
}

#ifdef CC_AST_HAS_VERSION_1_4
AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Flash for dgv trunk application");
#else
char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
#endif
